"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
from helpers import convertor
# from distutils.version import LooseVersion


def get_info(cluster_obj, node_list, xdr_port, user=None, password=None):

    info = dict()
    node_list = node_list.replace(' ', '')
    node_list = convertor.text_to_list(node_list, ',')
    for node_address in node_list:
        node_obj = cluster_obj.get_node(node_address)
        info[node_address] = dict()
        xdr_obj = None
        if node_obj is None or node_obj.is_valid() is False:
            info[node_address]['node_status'] = "off"
            info[node_address]['xdr_status'] = "off"
        else:
            info[node_address]['node_status'] = node_obj.get_status(user=user, password=password)
            xdr_obj = cluster_obj.get_xdr(xdr_port, node_address,
                                            user=user, password=password)
        if xdr_obj == {}:
            info[node_address]['xdr_status'] = "off"
        elif xdr_obj is not None:
            info[node_address]['xdr_status'] = "on"

            stats, status = xdr_obj.get_subset_of_statistics(
                    ['stat_recs_outstanding', 'timediff_lastship_cur_secs',
                     'esmt_bytes_shipped', 'stat_recs_relogged',
                     'stat_recs_shipped', 'free_dlog_pct', 'xdr_uptime',
                     'cur_throughput', 'esmt-bytes-shipped', 'free-dlog-pct',
                     'xdr-uptime'],
                    user=user, password=password
                )

            if stats['esmt-bytes-shipped'] == 'n/s':
                stats['esmt-bytes-shipped'] = stats['esmt_bytes_shipped']

            if stats['xdr-uptime'] == 'n/s':
                stats['xdr-uptime'] = stats['xdr_uptime']

            if stats['free-dlog-pct'] == 'n/s':
                stats['free-dlog-pct'] = stats['free_dlog_pct']

            # NOTE: Stat names were inconsistent in builds 3.7.0.2 and
            # 3.7.0-139-g8159f2c
            # xdr_uptime - > 3.7.0-139-g8159f2c
            # xdr-uptime -> 3.7.0.2
            # Fixing stat name issue with version comparison fails in this case
            # if LooseVersion(node_obj.get_build()) > LooseVersion('3.7.0'):
            #     stats, status = xdr_obj.get_subset_of_statistics(
            #         ['stat_recs_outstanding', 'timediff_lastship_cur_secs',
            #          'esmt_bytes_shipped', 'stat_recs_relogged',
            #          'stat_recs_shipped', 'free_dlog_pct', 'xdr_uptime',
            #          'cur_throughput'],
            #         user=user, password=password
            #     )
            #
            #     stats['esmt-bytes-shipped'] = stats.pop('esmt_bytes_shipped')
            #     stats['free-dlog-pct'] = stats.pop('free_dlog_pct')
            #     stats['xdr-uptime'] = stats.pop('xdr_uptime')
            #
            # else:
            #     stats, status = xdr_obj.get_subset_of_statistics(['stat_recs_outstanding',
            #         'timediff_lastship_cur_secs', 'esmt-bytes-shipped',
            #         'stat_recs_relogged', 'stat_recs_shipped', 'free-dlog-pct',
            #         'xdr-uptime', 'cur_throughput'], user=user, password=password)

            stats.update(info[node_address])
            info[node_address] = stats
            if status is 'off':
                node_obj.set_xdr()

    return info


def get_stats(cluster_obj, node_address, xdr_port, stats_type, user=None, password=None):

    #info = dict(address=node_address)
    info = dict()
    node_obj = cluster_obj.get_node(node_address)

    xdr_obj = None
    if node_obj is None or node_obj.is_valid() is False:
        info['node_status'] = "off"
        info['xdr_status'] = "off"
    elif node_obj.get_status(user=user, password=password) == "off":
        info['node_status'] = "off"
        xdr_obj = cluster_obj.get_xdr(xdr_port, node_address,
                                        user=user, password=password)
    else:
        info['node_status'] = "on"
        xdr_obj = cluster_obj.get_xdr(xdr_port, node_address,
                                        user=user, password=password)

    if xdr_obj == {}:
        info['xdr_status'] = "off"
    elif xdr_obj is not None:
        info['xdr_status'] = "on"
        stats = xdr_obj.get_statistics(stats_type, user=user, password=password)
        stats.update(info)
        configurations = xdr_obj.get_configurations(stats_type=stats_type, user=user, password=password)
        stats.update(configurations)
        info = stats
        info['node_status'] = node_obj.get_status(user=user, password=password)
    return info


def get_all_config(cluster_obj, node_address, xdr_port,user=None, password=None):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    xdr_obj = None
    if node_obj is None or node_obj.is_valid() is False:
        info['node_status'] = "off"
        info['xdr_status'] = "off"
    elif node_obj.get_status(user=user, password=password) == "off":
        info['node_status'] = "off"
        xdr_obj = cluster_obj.get_xdr(xdr_port, node_address,
                                        user=user, password=password)
    else:
        info['node_status'] = "on"
        xdr_obj = cluster_obj.get_xdr(xdr_port, node_address,
                                        user=user, password=password)

    if xdr_obj == {}:
        info['xdr_status'] = "off"
    elif xdr_obj is not None:
        info['xdr_status'] = "on"
        configurations = xdr_obj.get_configurations(user=user, password=password)
        configurations.update(info)
        info = configurations
    return info


def change_xdr_status(desired_status, cluster_obj, node_address,
                            username="", password=None, ssh_key = None, usr=None, passwd=None):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    xdr_obj = None
    if node_obj is not None and node_obj.get_status(user=usr, password=passwd) is not "off" and node_obj.is_valid() is True:
        xdr_obj = cluster_obj.get_xdr("3004", node_address,
                                        user=usr, password=passwd)
    if xdr_obj is None:
        info['node_status'] = "off"
    elif xdr_obj == {} and desired_status == "off":
        info['status'] = "Failue"
        info['error'] = "XDR Already Off"
    elif xdr_obj is not None and xdr_obj != {} and desired_status == "on":
        info['status'] = "Failue"
        info['error'] = "XDR Already On"
    else:
        status = node_obj.change_xdr_status(desired_status, username, password, ssh_key)
        info.update(status)
    return info
